* create an objectg to do some processing
oE = CREATEOBJECT('eventprocessor')
oO = CREATEOBJECT('processor')

oX = NEWOBJECT('logger')
oY = NEWOBJECT('progress')

oE.BindEvent(oO, 'STRINGCREATED', oX, 'Update', .25, 'After_Update')
oE.BindEvent(oO, 'FILEONDISK', oY, 'Update', .25, 'After_Update')

seconds = SECONDS()
oO.Process()
ACTIVATE SCREEN 
?SECONDS() - seconds

DEFINE CLASS processor AS Session

	Iterations = 500
	fileName = 'file.txt'
	fileContents = ''

	FUNCTION Process

		* iterate the specified number of times
		LOCAL iteration
		FOR iteration = 1 TO this.iterations
		
			* create a file
			this.createFileContents()
			oE.RaiseEvent(this, 'STRINGCREATED', TRANSFORM(iteration))
			
			* add it to disk
			this.sendFileToDisk()
			oE.RaiseEvent(this, 'FILEONDISK', TRANSFORM(iteration))
			
			* delete the file
			this.deleteFile()
			oE.DoEvents()
		ENDFOR 
		oE.DoEvents(.T.)
	RETURN 

	FUNCTION createFileContents
	
		* add a random string to another string 1000 times
		this.fileContents = ''
		LOCAL i
		FOR i = 1 TO 1000
			this.fileContents = this.fileContents + SYS(2015)
		ENDFOR 
	RETURN 

	FUNCTION sendFileToDisk
		STRTOFILE(this.fileContents, this.fileName)
	RETURN 
	
	FUNCTION deleteFile
		ERASE (this.fileName)
	RETURN 

ENDDEFINE

DEFINE CLASS eventProcessor AS custom

	DIMENSION queuedEvents[1, 3]
	DIMENSION delegates[1, 3]
	eventCount = 0
	delegateCount = 0

	FUNCTION BindEvent(object, event, eventHandler, delegate, delay, afterBatch)
		WITH this
			.delegateCount = .delegateCount + 1
			DIMENSION .delegates[.delegateCount, 8]
			.delegates[.delegateCount, 1] = object
			.delegates[.delegateCount, 2] = event
			.delegates[.delegateCount, 3] = eventHandler
			.delegates[.delegateCount, 4] = delegate
			.delegates[.delegateCount, 5] = IIF(EMPTY(delay), 0, delay)
			.delegates[.delegateCount, 6] = 0 && Next run
			.delegates[.delegateCount, 7] = SYS(2015) && delegateId
			.delegates[.delegateCount, 8] = afterBatch
		ENDWITH 
	RETURN 

	FUNCTION RaiseEvent(object, event, parameters)

		IF EMPTY(parameters)
			parameters = ''
		ENDIF 
		event = UPPER(event)

		WITH this 

			FOR i = 1 TO .delegateCount
				IF .delegates[i, 2] == event AND .delegates[i, 1] = object

					* if this there is a value in the delay column, queue the event for a further time
					IF .delegates[i, 5] > 0
						.queueEvent(.delegates[i, 7], .delegates[i, 3], .delegates[i, 4], parameters)
						
						* If there is no 6th element (the next run), set that now
						IF .delegates[i, 6] = 0
							.delegates[i, 6] = SECONDS() + .delegates[i, 5]
						ENDIF 
						
					ELSE 
						* otherwise handle it immediately
						.handleEvent(.delegates[i, 3], .delegates[i, 4], parameters)
					ENDIF 
				ENDIF 
			ENDFOR 

		ENDWITH 
	RETURN 
		
	PROTECTED FUNCTION handleEvent(object, delegate, parameters)
		LOCAL expression
		expression = 'object.' + delegate + '(' + parameters + ')'
		EVALUATE(expression)
	RETURN 

	PROTECTED FUNCTION queueEvent(delegateId, object, delegate, parameters)
		WITH this 

			* add this to the queued event stack
			.eventCount = .eventCount + 1
			DIMENSION .queuedEvents[.eventCount, 4]
			.queuedEvents[.eventCount, 1] = delegateId
			.queuedEvents[.eventCount, 2] = object
			.queuedEvents[.eventCount, 3] = delegate
			.queuedEvents[.eventCount, 4] = parameters

		ENDWITH 
	RETURN 

	FUNCTION DoEvents(forceEvents)
		LOCAL i, seconds
		seconds = SECONDS()

		* when the timer fires, it should know what events to call
		WITH this

			FOR i = 1 TO .delegateCount
				IF forceEvents OR .delegates[i, 6] <= seconds

					* Handle all the events queued for the deletegate
					.doEventsForDelegate(.delegates[i, 7])
					.delegates[i, 6] = 0

					IF NOT EMPTY(.delegates[i, 8])
						.handleEvent(.delegates[i, 3], .delegates[i, 8], '')
					ENDIF 

				ENDIF 
			ENDFOR 

		ENDWITH 
	RETURN 

	PROTECTED FUNCTION doEventsForDelegate(delegateId)
		LOCAL i, j
		j = 1
		FOR i = 1 TO .eventCount
			IF .queuedEvents[j, 1] == delegateID
				.handleEvent(.queuedEvents[j, 2], .queuedEvents[j, 3], .queuedEvents[j, 4])
				.removeQueuedEvent(j)
			ELSE 
				j = j + 1 
			ENDIF 
		ENDFOR 
	RETURN 
	
	PROTECTED FUNCTION removeQueuedEvent(eventIndex)
		WITH this
			ADEL(.queuedEvents, eventIndex)
			.eventCount = .eventCount - 1
			IF .eventCount > 0
				DIMENSION .queuedEvents[.eventCount, 4]
			ENDIF 
		ENDWITH 
	RETURN 

ENDDEFINE

DEFINE CLASS logger AS Session OLEPUBLIC

	iteration = 0
	changed = .F.
	oForm = .NULL.
	
	FUNCTION Init
		this.oForm = CREATEOBJECT('progressform', 'String Created')
		this.oForm.Left = 200
		this.oForm.Show()
	RETURN 
	FUNCTION Destroy
		this.oForm = .NULL.
	RETURN 

	FUNCTION Update
		LPARAMETERS iteration
		this.iteration = iteration
	RETURN 
	FUNCTION After_Update
		this.oForm.counter.value = this.iteration
	RETURN 
ENDDEFINE

DEFINE CLASS progress AS Session OLEPUBLIC

	iteration = 0
	oForm = .NULL.

	FUNCTION Init
		this.oForm = CREATEOBJECT('progressform', 'File Created')
		this.oForm.Left = 0
		this.oForm.Show()
	RETURN 
	FUNCTION Destroy
		this.oForm = .NULL.
	RETURN 

	FUNCTION Update
		LPARAMETERS iteration
		this.iteration = iteration
	RETURN 
	FUNCTION After_Update
		this.oForm.counter.value = this.iteration
	RETURN 

ENDDEFINE

DEFINE CLASS progressform AS form 

	Height = 55
	Width = 175

	ADD OBJECT lblCaption AS label WITH ;
		autoSize = .T.
	ADD OBJECT counter AS textbox WITH ;
		top = 30

	FUNCTION Init
		LPARAMETERS cCaption
		this.lblCaption.Caption = cCaption
	RETURN 

ENDDEFINE
